@@ -617,6 +617,9 @@ int controller::run(int argc, char * argv[]) {
 	} catch (const dbexception& e) {
 		std::cerr << strprintf::fmt(_("Error: opening the cache file `%s' failed: %s"), cache_file, e.what()) << std::endl;
 		return EXIT_FAILURE;
+	} catch (const std::runtime_error& e) {
+		std::cerr << strprintf::fmt(_("Error: %s"), e.what()) << std::endl;
+		return EXIT_FAILURE;
 	}
 
 	if (!silent) {
@@ -740,15 +743,20 @@ int controller::run(int argc, char * argv[]) {
 
 	// if configured, we fill all query feeds with some data; no need to sort it, it will be refilled when actually opening it.
 	if (cfg.get_configvalue_as_bool("prepopulate-query-feeds")) {
-		std::cout << _("Prepopulating query feeds...");
-		std::cout.flush();
+		if (! do_export && ! silent) {
+			std::cout << _("Prepopulating query feeds...");
+			std::cout.flush();
+		}
 		std::lock_guard<std::mutex> feedslock(feeds_mutex);
 		for (auto feed : feeds) {
 			if (feed->rssurl().substr(0,6) == "query:") {
 				feed->update_items(get_all_feeds_unlocked());
 			}
 		}
-		std::cout << _("done.") << std::endl;
+
+		if (!do_export && ! silent) {
+			std::cout << _("done.") << std::endl;
+		}
 	}
 
 	sort_feeds();
@@ -1150,13 +1158,16 @@ void controller::print_version_information(const char * argv0, unsigned int leve
 	if (level<=1) {
 		std::cout << PROGRAM_NAME << " " << PROGRAM_VERSION << " - " << PROGRAM_URL << std::endl;
 		std::cout << "Copyright (C) 2006-2015 Andreas Krennmair" << std::endl;
-		std::cout << "Copyright (C) 2015-2017 Alexander Batischev" << std::endl;
+		std::cout << "Copyright (C) 2015-2018 Alexander Batischev" << std::endl;
 		std::cout << "Copyright (C) 2006-2017 Newsbeuter contributors" << std::endl;
-		std::cout << "Copyright (C) 2017 Newsboat contributors" << std::endl;
+		std::cout << "Copyright (C) 2017-2018 Newsboat contributors" << std::endl;
 		std::cout << std::endl;
 
-		std::cout << _("newsboat is free software and licensed under the MIT License.") << std::endl;
-		std::cout << strprintf::fmt(_("Type `%s -vv' for more information."), argv0) << std::endl << std::endl;
+		std::cout << strprintf::fmt(
+				_("Newsboat is free software licensed under the MIT License. (Type `%s -vv' to see the full text.)"),
+				argv0) << std::endl;
+		std::cout << _("It bundles JSON for Modern C++ library, licensed under the MIT License: https://github.com/nlohmann/json") << std::endl;
+		std::cout << std::endl;
 
 		struct utsname xuts;
 		uname(&xuts);
