@@ -57,6 +57,11 @@ static op_desc opdescs[] = {
 		_("Mark all feeds read"),
 		KM_FEEDLIST
 	},
+	{
+		OP_MARKALLABOVEASREAD, "mark-all-above-as-read", "",
+		_("Mark all above as read"),
+		KM_ARTICLELIST
+	},
 	{
 		OP_SAVE, "save", "s",
 		_("Save article"),
