@@ -134,7 +134,11 @@ std::vector<tagged_feedurl> oldreader_api::get_subscribed_urls() {
 
 			json_object_object_get_ex(sub, "categories", &node);
 			struct array_list * categories = json_object_get_array(node);
-			for (int i = 0; i < array_list_length(categories); i++) {
+#if JSON_C_MAJOR_VERSION == 0 && JSON_C_MINOR_VERSION < 13
+		for (int i = 0; i < array_list_length(categories); i++) {
+#else
+		for (size_t i = 0; i < array_list_length(categories); i++) {
+#endif
 				json_object* cat = json_object_array_get_idx(node, i);
 				json_object* label_node {};
 				json_object_object_get_ex(cat, "label", &label_node);
@@ -167,7 +171,12 @@ void oldreader_api::add_custom_headers(curl_slist** custom_headers) {
 bool oldreader_api::mark_all_read(const std::string& feedurl) {
 	std::string real_feedurl = feedurl.substr(strlen(OLDREADER_FEED_PREFIX), feedurl.length() - strlen(OLDREADER_FEED_PREFIX));
 	std::vector<std::string> elems = utils::tokenize(real_feedurl, "?");
-	real_feedurl = utils::unescape_url(elems[0]);
+	try {
+		real_feedurl = utils::unescape_url(elems[0]);
+	} catch (const std::runtime_error& e) {
+		LOG(level::DEBUG, "oldreader_api::mark_all_read: Failed to unescape_url(%s): %s", elems[0], e.what());
+		return false;
+	}
 	std::string token = get_new_token();
 
 	std::string postcontent = strprintf::fmt("s=%s&T=%s", real_feedurl, token);
