@@ -379,11 +379,17 @@ std::string utils::retrieve_url(
 		const std::string& url,
 		configcontainer * cfgcont,
 		const std::string& authinfo,
-		const std::string* postdata)
+		const std::string* postdata,
+		CURL* cached_handle)
 {
 	std::string buf;
 
-	CURL * easyhandle = curl_easy_init();
+	CURL* easyhandle;
+	if (cached_handle) {
+		easyhandle = cached_handle;
+	} else {
+		easyhandle = curl_easy_init();
+	}
 	set_common_curl_options(easyhandle, cfgcont);
 	curl_easy_setopt(easyhandle, CURLOPT_URL, url.c_str());
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEFUNCTION, my_write_data);
@@ -401,7 +407,9 @@ std::string utils::retrieve_url(
 	}
 
 	curl_easy_perform(easyhandle);
-	curl_easy_cleanup(easyhandle);
+	if (!cached_handle) {
+		curl_easy_cleanup(easyhandle);
+	}
 
 	if (postdata != nullptr) {
 		LOG(level::DEBUG, "utils::retrieve_url(%s)[%s]: %s", url, postdata, buf);
@@ -947,6 +955,11 @@ void utils::set_common_curl_options(CURL * handle, configcontainer * cfg) {
 		curl_easy_setopt(handle, CURLOPT_COOKIEFILE, cookie_cache.c_str());
 		curl_easy_setopt(handle, CURLOPT_COOKIEJAR, cookie_cache.c_str());
 	}
+
+	const char* curl_ca_bundle = ::getenv("CURL_CA_BUNDLE");
+	if (curl_ca_bundle != nullptr) {
+		curl_easy_setopt(handle, CURLOPT_CAINFO, curl_ca_bundle);
+	}
 }
 
 std::string utils::get_content(xmlNode * node) {
@@ -1007,11 +1020,29 @@ curl_proxytype utils::get_proxy_type(const std::string& type) {
 }
 
 std::string utils::escape_url(const std::string& url) {
-	return replace_all(replace_all(url,"?","%3F"), "&", "%26");
+	CURL * easyhandle = curl_easy_init();
+	char *output = curl_easy_escape(easyhandle, url.c_str(), 0);
+	if (!output) {
+		LOG(level::DEBUG, "Libcurl failed to escape url: %s", url);
+		throw std::runtime_error("escaping url failed");
+	}
+	std::string s = output;
+	curl_free(output);
+	curl_easy_cleanup(easyhandle);
+	return s;
 }
 
 std::string utils::unescape_url(const std::string& url) {
-	return replace_all(replace_all(url,"%3F","?"), "%26", "&");
+	CURL * easyhandle = curl_easy_init();
+	char *output = curl_easy_unescape(easyhandle, url.c_str(), 0, NULL);
+	if (!output) {
+		LOG(level::DEBUG, "Libcurl failed to escape url: %s", url);
+		throw std::runtime_error("escaping url failed");
+	}
+	std::string s = output;
+	curl_free(output);
+	curl_easy_cleanup(easyhandle);
+	return s;
 }
 
 std::wstring utils::clean_nonprintable_characters(std::wstring text) {
@@ -1149,14 +1180,18 @@ void utils::remove_soft_hyphens(std::string& text) {
 }
 
 bool utils::is_valid_podcast_type(const std::string& mimetype) {
+	const std::regex acceptable_rx {
+		"(audio|video)/.*",
+		std::regex_constants::ECMAScript | std::regex_constants::optimize };
+
 	const std::unordered_set<std::string> acceptable = {
-		"audio/mpeg",
-		"audio/mp3",
-		"audio/x-mp3",
-		"audio/ogg",
 		"application/ogg"
 	};
-	return acceptable.find(mimetype) != acceptable.end();
+
+	const bool found = acceptable.find(mimetype) != acceptable.end();
+	const bool matches = std::regex_match(mimetype, acceptable_rx);
+
+	return found || matches;
 }
 
 
